/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.dao;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.LockModeType;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.validation.constraints.NotNull;

import almira.sample.domain.DomainObject;


/**
 * Generic DAO implementation.
 * <br/>
 * [Bauer07] page 727<br/>
 * <br/>
 * See also:
 * http://blog.springsource.com/2006/08/07/using-jpa-in-spring-without-
 * referencing-spring/
 * <p>
 * Note that the class is abstract because the constructor gets the type
 * arguments from the superclass.
 * </p>
 *
 * @param <T>  type of elements managed by this DAO
 * @param <ID> the generic type
 */
abstract class AbstractDaoHibernate<T extends DomainObject<ID>, ID extends Serializable>
        implements Dao<T, ID> {

    /**
     * The persistent class.
     */
    private final Class<T> entityBeanType;

    /**
     * The entity manager.
     */
    @PersistenceContext
    private EntityManager em;

    public void setEntityManager(EntityManager entityManager) {
        this.em = entityManager;
    }

    EntityManager getEntityManager() {
        return em;
    }

    /**
     * Constructor.
     * <p/>
     * [Bauer2007] page 711
     */
    @SuppressWarnings("unchecked")
    AbstractDaoHibernate() {
        this.entityBeanType = (Class<T>) ((ParameterizedType) getClass()
                .getGenericSuperclass()).getActualTypeArguments()[0];
    }

    /**
     * {@inheritDoc}
     */
    // http://docs.jboss.org/hibernate/entitymanager/3.5/reference/en/html/objectstate.html
    // http://stackoverflow.com/questions/1069992/jpa-entitymanager-why-use-persist-over-merge
    @Override
    public T makePersistent(@NotNull T entity) {
        if (entity.getId() == null) {
            em.persist(entity);
            return entity;
        } else {
            return em.merge(entity);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public T findById(ID id, boolean lock) {

        T entity = em.find(entityBeanType, id);
        if (lock) {
            // http://en.wikibooks.org/wiki/Java_Persistence/Locking#JPA_2.0_Locking
            // [BAUER07] page 467, 712
            em.lock(entity, LockModeType.OPTIMISTIC);
        }
        return entity;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    @Override
    public T findByName(String name) {
        Query query = em.createNamedQuery("findCatapultByName");
        query.setParameter("name", name);
        return (T) query.getSingleResult();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void makeTransient(T entity) {
        em.remove(entity);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    @SuppressWarnings("unchecked")
    public List<T> findAll(final int first, final int count) {

        // named queries must be defined in catapult-api (orm.xml)
        Query query = em.createNamedQuery("findAllCatapults")
                        .setFirstResult(first)
                        .setMaxResults(count);

        // use query result cache. [Bauer07] page 691
        query.setHint("org.hibernate.cacheable", true);

        return query.getResultList();
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public long count() {
        // [Bauer07] page 627
        return (Long) em.createNamedQuery("countAllCatapults").getSingleResult();
    }

    @Override
    public void flush() {
        em.flush();
    }

    @Override
    public void clear() {
        em.clear();
    }
}
