/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.dao;

import java.io.Serializable;
import java.util.List;

import almira.sample.domain.DomainObject;


/**
 * Generic Data Access Object interface.<br>
 * Inspired from [McKenzie08], page 389 and [Bauer07] page 712.
 *
 * @param <T>  type of elements managed by this DAO
 * @param <ID> the generic type
 */
public interface Dao<T extends DomainObject<ID>, ID extends Serializable> {

    /**
     * Persist the object of Class T.
     *
     * @param entity the Object to persist
     * @return the persisted entityClass
     */
    T makePersistent(T entity);

    /**
     * Load persistent object from the database.
     *
     * @param id the ID of the object to retrieve
     * @return the object from the database
     */
    T findById(ID id, boolean lock);

    /**
     * Load persistent object from the database.
     *
     * @param name the name of the object to retrieve
     * @return the object from the database
     */
    T findByName(String name);

    /**
     * Deletes the object of Class T.
     *
     * @param entity the Object to delete
     */
    void makeTransient(T entity);

    /**
     * Find count records of Class T from starting at first.
     *
     * @param startIndex the start index
     * @param fetchSize  the fetch size
     * @return an Iterator of class T
     */
    List<T> findAll(final int startIndex, final int fetchSize);

    /**
     * Counts the number or records.
     *
     * @return the number of objects
     */
    long count();

    void flush();

    void clear();
}
