/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.dao;

import static org.fest.assertions.Assertions.assertThat;
import static org.junit.Assert.assertEquals;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.LockModeType;
import javax.persistence.Query;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import almira.sample.dao.AbstractDaoHibernate;
import almira.sample.dao.CatapultDaoHibernate;
import almira.sample.domain.Catapult;
import almira.sample.testing.CatapultBuilder;

import com.mycila.testing.junit.MycilaJunitRunner;
import com.mycila.testing.plugin.jmock.Mock;
import com.mycila.testing.plugin.jmock.MockContext;


@RunWith(MycilaJunitRunner.class)
public class DaoHibernateTest {

    @MockContext private Mockery context;
    @Mock private EntityManager em;
    @Mock private Query query;

    private static final String CATAPULT_NAME = "test";
    private transient AbstractDaoHibernate<Catapult, Long> dao = null;

    @Before
    public void setUp() {
        dao = new CatapultDaoHibernate();
        dao.setEntityManager(em);
    }

    @Test
    public void getEntityManagerReturnEntityManager() {
        assertEquals(em, dao.getEntityManager());
    }

    @Test
    public void makePersistentPersists() {
        final Catapult catapult = new CatapultBuilder(null, CATAPULT_NAME).build();
        context.checking(new Expectations() {{ oneOf(em).persist(catapult); }} );
        dao.makePersistent(catapult);
    }

    @Test
    public void makePersistentMerges() {
        final Catapult catapult = new CatapultBuilder(1L, CATAPULT_NAME).build();
        context.checking(new Expectations() {{ oneOf(em).merge(catapult); will(returnValue(catapult)); }} );
        dao.makePersistent(catapult);
    }

    @Test
    public void makeTransientRemoves() {
        final Catapult catapult = new CatapultBuilder(1L, CATAPULT_NAME).build();
        context.checking(new Expectations() {{ oneOf(em).remove(catapult); }} );
        dao.makeTransient(catapult);
    }

    @Test
    public void findByIdFinds() {
        final Catapult catapult = new CatapultBuilder(1L, CATAPULT_NAME).build();
        context.checking(new Expectations() {{ oneOf(em).find(Catapult.class, 1L); will(returnValue(catapult)); }} );
        final Catapult actual = dao.findById(1L, false);
        assertThat(actual).isEqualTo(catapult);
    }

    @Test
    public void findByNameFinds() {
        final Catapult catapult = new CatapultBuilder(1L, CATAPULT_NAME).build();
        context.checking(new Expectations() {{
            oneOf(em).createNamedQuery("findCatapultByName"); will(returnValue(query));
            oneOf(query).setParameter("name", CATAPULT_NAME);
            oneOf(query).getSingleResult(); will(returnValue(catapult));
        }} );
        final Catapult actual = dao.findByName(CATAPULT_NAME);
        assertThat(actual).isEqualTo(catapult);
    }

    @Test
    public void findByIdFindsAndLocks() {
        final Catapult catapult = new CatapultBuilder(1L, CATAPULT_NAME).build();
        context.checking(new Expectations() {{
            oneOf(em).find(Catapult.class, 1L); will(returnValue(catapult));
            oneOf(em).lock(catapult, LockModeType.OPTIMISTIC);
        }} );
        final Catapult actual = dao.findById(1L, true);
        assertThat(actual).isEqualTo(catapult);
    }

    @Test
    public void findAllFinds() {
        final List<Catapult> result = new CatapultBuilder(1L, CATAPULT_NAME).asList();
        context.checking(new Expectations() {{
            oneOf(em).createNamedQuery("findAllCatapults"); will(returnValue(query));
            oneOf(query).setFirstResult(0); will(returnValue(query));
            oneOf(query).setMaxResults(1); will(returnValue(query));
            oneOf(query).setHint("org.hibernate.cacheable", true);
            oneOf(query).getResultList(); will(returnValue(result));
        }} );
        final List<Catapult> actual = dao.findAll(0, 1);
        assertThat(actual).isEqualTo(result);
    }

    @Test
    public void countCounts() {
        context.checking(new Expectations() {{
            oneOf(em).createNamedQuery("countAllCatapults"); will(returnValue(query));
            oneOf(query).getSingleResult(); will(returnValue(1L));
        }} );
        assertThat(dao.count()).isEqualTo(1L);
    }

    @Test
    public void flushFlushes() {
        context.checking(new Expectations() {{ oneOf(em).flush(); }} );
        dao.flush();
    }

    @Test
    public void clearClears() {
        context.checking(new Expectations() {{ oneOf(em).clear(); }} );
        dao.clear();
    }
}
