/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.dao;

import static org.fest.assertions.Assertions.assertThat;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.dbunit.dataset.datatype.DataType;
import org.dbunit.dataset.datatype.DataTypeException;
import org.dbunit.dataset.datatype.TimestampDataType;
import org.junit.Test;

// http://stackoverflow.com/questions/7159895/simpledateformat-is-not-parsing-the-milliseconds-correctly
// http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4516915

public class DbUnitTimestampParsingTest {

    // Magic numbers
    private static final int MILLIS_140 = 140;
    private static final int MILLIS_14 = 14;
    private static final int YEAR = 2011;
    private static final int SECONDS = 13;
    private static final int MINUTES = 12;
    private static final int HOURS = 11;
    private static final int DAY = 6;

    // java.sql.Timestamp
    private static final String PADDING = "00000000000000000000000";
    private static final String TIMESTR = "2011-12-06 11:12:13.14";

    private static final Timestamp TS = Timestamp.valueOf(TIMESTR);
    private static final int NANOS = 140000000;

    // java.util.Date
    private static final SimpleDateFormat DF = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");

    private static final GregorianCalendar CALENDAR14 = setCalendar14();
    private static final String DATESTR14 = "2011-12-06 11:12:13.014";
    private static GregorianCalendar setCalendar14() {
        GregorianCalendar c = new GregorianCalendar(YEAR, Calendar.DECEMBER, DAY, HOURS, MINUTES, SECONDS);
        c.add(Calendar.MILLISECOND, MILLIS_14);
        return c;
    }

    private static final GregorianCalendar CALENDAR140 = setCalendar140();
    private static final String DATESTR140 = "2011-12-06 11:12:13.140";
    private static GregorianCalendar setCalendar140() {
        GregorianCalendar c = new GregorianCalendar(YEAR, Calendar.DECEMBER, DAY, HOURS, MINUTES, SECONDS);
        c.add(Calendar.MILLISECOND, MILLIS_140);
        return c;
    }

    // --------- Timestamp manipulation ---------
    @Test
    public void timestampParsesStringCorrectly() {
        assertThat(TS.toString()).isEqualTo(TIMESTR);
        assertThat(TS.getNanos()).isEqualTo(NANOS);
    }

    @Test
    public void dbUnitTypeCastOnTimestamp() throws DataTypeException {
        checkTypeCast(TS);
    }

    @Test
    public void dbUnitTypeCastOnString() throws DataTypeException {
        checkTypeCast(TIMESTR);
    }

    // A workaround is to reformat the time as a Date, so milliseconds will
    // be printed as ".140" == 140 000 microseconds == 140 000 000 nanos.
    @Test
    public void dBUnitDateFormatWorkaround1() throws DataTypeException {
        Date date = new Date(TS.getTime());
        String dateString = DF.format(date);
        checkTypeCast(dateString);
    }

    // We can also right-pad the Timestamp string to make it look like a Date
    @Test
    public void dBUnitDateFormatWorkaround2() throws DataTypeException {
        checkTypeCast(paddRight(TIMESTR));
    }

    // --------- Date manipulation ---------
    @Test
    public void dateFormatFormatsCorrectly() {
        Date date = new Date(CALENDAR14.getTimeInMillis());
        assertThat(DF.format(date)).isEqualTo(DATESTR14);

        date = new Date(CALENDAR140.getTimeInMillis());
        assertThat(DF.format(date)).isEqualTo(DATESTR140);
    }

    // Confirm that it works with a Date object
    @Test
    public void dbUnitTypeCastOnDate() throws DataTypeException {
        Date date = new Date(CALENDAR140.getTimeInMillis());
        checkTypeCast(date);
    }

    // Confirm that ".140" is interpreted as 140 000 000 nanos
    @Test
    public void dbUnitTypeCastOnDateString() throws DataTypeException {
        checkTypeCast(DATESTR140);
    }

    // --------- Helper methods ---------
    private String paddRight(String string) {
        return string + PADDING.substring(string.length());
    }

    private void checkTypeCast(Object object) throws DataTypeException {
        Timestamp ts = typeCast(object);
        assertThat(ts.toString()).isEqualTo(TIMESTR);
        assertThat(ts.getNanos()).isEqualTo(NANOS);
    }

    private Timestamp typeCast(Object object) throws DataTypeException {
        DataType dt = TimestampDataType.forSqlType(java.sql.Types.TIMESTAMP);
        return (Timestamp) dt.typeCast(object);
    }
}
