/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.dao;

import static org.fest.assertions.Assertions.assertThat;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.Date;

import org.junit.Test;
import org.springframework.test.context.transaction.AfterTransaction;
import org.springframework.transaction.annotation.Transactional;


import almira.sample.dao.AbstractSpringTransactionalTestCase;
import almira.sample.domain.Catapult;


public class TSCEntityPersistence extends AbstractSpringTransactionalTestCase {

    private static Long staticId;

    @Test
    @Transactional(readOnly = false)
    public void persist() {
        Catapult catapult = new Catapult();
        catapult.setName("Persisted");
        em.persist(catapult);
        em.flush(); // send actual SQL to DB

        staticId = catapult.getId();
        assertNotNull("id should not be null after persist()", staticId);
    }

    @AfterTransaction
    public void checkPostConditions() {
        Catapult c = em.find(Catapult.class, staticId);
        assertThat(c).isNotNull();
        assertThat(c.getCreated()).isNotNull();
        assertThat(c.getLastModified()).isNotNull();
        assertThat(c.getVersion()).isEqualTo(0);
    }

    @Test
    @Transactional(readOnly = false)
    public void hidratedDateIsDateNotTimestamp() {
        Catapult catapult = new Catapult();
        catapult.setName("WithDate");
        em.persist(catapult);
        em.flush(); // send actual SQL to DB

        Long id = catapult.getId();
        assertNotNull("id should not be null after persist()", id);
        System.out.println("Persisted catapult with id=" + id);

        // detach so object must be reloaded from the database
        em.detach(catapult);

        // reload as a different object
        Catapult c = em.find(Catapult.class, id);

        // sanity check: make sure it's a different object instance
        assertThat(c).isNotNull();
        assertTrue("loaded catapult should be a different instance", c != catapult); // NOPMD
        assertEquals(Date.class, c.getCreated().getClass());
    }

    @Test
    public void hashcodeIsSameAfterPersisting() {
        Catapult cata = new Catapult();
        cata.setName("Catapult");
        int h1 = cata.hashCode();
        em.persist(cata);
        int h2 = cata.hashCode();
        assertTrue(h1 == h2);
    }
}
