/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.fest.reflect.core.Reflection;

/**
 * Helper class to build Domain Objects for testing purposes.
 *
 * See also: [Bloch08] page 14
 * @author 'Daniel TISCHER'
 */
public class ObjectBuilder<T> {

    private T object;

    public ObjectBuilder(Class<T> clazz, Long id) {
        try {
            object = clazz.newInstance();
        }
        catch (Exception e) {
            throw new IllegalStateException("Can't instantiate object", e);
        }
        Reflection.field("id").ofType(Long.class).in(object).set(id);
    }

    public ObjectBuilder<T> version(int version) {
        Reflection.field("version").ofType(Integer.class).in(object).set(version);
        return this;
    }

    public ObjectBuilder<T> created(Date created) {
        Reflection.field("created").ofType(Date.class).in(object).set(created);
        return this;
    }

    public ObjectBuilder<T> lastModified(Date lastModified) {
        Reflection.field("lastModified").ofType(Date.class).in(object).set(lastModified);
        return this;
    }

    /**
     * Call setter on field. The setter method name is deduced from the field
     * name, and the method called via reflection.
     *
     * @param field the field to set.
     * @param value the value to the set the field to.
     * @return the builder.
     */
    public ObjectBuilder<T> set(String field, Object value) {
        String methodName = "set" + ("" + field.charAt(0)).toUpperCase(Locale.ENGLISH) + field.substring(1);
        Reflection.method(methodName).withParameterTypes(value.getClass()).in(object).invoke(value);
        return this;
    }

    public T build() {
        return object;
    }

    public List<T> asList() {
        final ArrayList<T> list = new ArrayList<T>();
        list.add(object);
        return list;
    }
}
