/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.web;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.logging.Logger;

import org.apache.wicket.ConverterLocator;
import org.apache.wicket.IConverterLocator;
import org.apache.wicket.Page;
import org.apache.wicket.Session;
import org.apache.wicket.protocol.http.WebApplication;
import org.apache.wicket.protocol.https.HttpsConfig;
import org.apache.wicket.protocol.https.HttpsMapper;
import org.apache.wicket.spring.injection.annot.SpringComponentInjector;
import org.apache.wicket.util.convert.converter.DateConverter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * Wicket application.
 */
@Component
public class MainApplication extends WebApplication {

    private static final Logger LOG = Logger
                    .getLogger(MainApplication.class.getName());

    /**
     * Version number of the application.
     */
    private String version;

    @Value("${rt.server.port.http}")
    private int httpPort;

    @Value("${rt.server.port.https}")
    private int httpsPort;

    private static final DateConverter DATE_CONVERTER = new DateConverter() {
        @Override
        public DateFormat getDateFormat(Locale locale) {
            return new SimpleDateFormat("dd-MM-yyyy HH:mm:ss.mmm", locale);
        }
    };

    /**
     * Constructor.
     */
    public MainApplication() {
        super();
        setVersion();
    }

    /**
     * Sets the version of the application from MANIFEST.MF in the jar that
     * packages the classes.<br/>
     * You MUST configure maven-war-plugin with
     * <code>&lt;archiveClasses&gt;true&lt;/archiveClasses&gt;</code>
     */
    private void setVersion() {
        final String packageVersion = getClass().getPackage().getImplementationVersion();
        if (packageVersion != null) {
            version = packageVersion;
        }
    }

    @Override
    protected final void init() {
        super.init();

        // Template method for overriding in a unit test
        initSpringInjector();

        // Cut page sizes in half for large HTML pages
        getMarkupSettings().setCompressWhitespace(true);
        getMarkupSettings().setStripComments(true);

        // https://cwiki.apache.org/WICKET/request-mapping.html
        mountPage("/admin", AdminPage.class);

        // Cargo expects index.html in <pingURL>
        mountPage("/index.html", IndexPage.class);

        if (httpPort == 0 || httpsPort == 0) {
            LOG.warning("HTTPS redirect not configured: ${rt.server.port.http}="
                        + httpPort + " ${rt.server.port.https}=" + httpsPort);
        }
        else {
            setRootRequestMapper(new HttpsMapper(getRootRequestMapper(),
                            new HttpsConfig(httpPort, httpsPort)));

            LOG.info("HTTP port configured for MainApplication: " + httpPort);
            LOG.info("HTTPS port configured for MainApplication: " + httpsPort);
        }
    }

    /**
     * [Dashorst09], page 310.
     * <p/>
     * See also:
     * http://apache-wicket.1842946.n4.nabble.com/SpringBean-and-serialization-td1872399.html
     */
    void initSpringInjector() {
        getComponentInstantiationListeners().add(new SpringComponentInjector(this));
    }

    @Override
    public final Class<? extends Page> getHomePage() {
        return IndexPage.class;
    }

    @Override
    protected final IConverterLocator newConverterLocator() {
        final ConverterLocator converterLocator = new ConverterLocator();
        converterLocator.set(Date.class, DATE_CONVERTER);
        return converterLocator;
    }

    @Override
    public final Session newSession(org.apache.wicket.request.Request request,
                    org.apache.wicket.request.Response response) {
        Session session = super.newSession(request, response);

        // make sure we get rid of 'country' to match keys in the language
        // choice component
        session.setLocale(new Locale(session.getLocale().getLanguage()));

        return session;
    }

    /**
     * Returns the version of the application.
     *
     * @return the version of the application.
     */
    public final String getVersion() {
        return version;
    }
}
