/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.web;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.navigation.paging.PagingNavigator;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.spring.injection.annot.SpringBean;

import almira.sample.domain.Catapult;
import almira.sample.service.ObjectQueryService;
import almira.sample.service.ObjectUpdateService;
import almira.sample.web.model.CatapultListDataProvider;
import almira.sample.web.model.CatapultSearchDataProvider;


/**
 * Shows result list of catapults matching the criteria for the search query, or
 * all catapults if the query is null.
 */
public class SearchResultPanel extends Panel {

    private static final long serialVersionUID = 1L;
    private static final int ITEMS_PER_PAGE = 5;

    @SpringBean
    private static ObjectUpdateService<Catapult, Long> catapultUpdateService;

    @SpringBean
    private static ObjectQueryService<Catapult, Long> catapultQueryService;

    /**
     * Constructor.
     *
     * @param id          component id
     * @param searchQuery the search query
     */

    @edu.umd.cs.findbugs.annotations.SuppressWarnings(
      value = {"SE_INNER_CLASS", "SIC_INNER_SHOULD_BE_STATIC_ANON"},
      justification = "This is the Wicket way")

    public SearchResultPanel(String id, String searchQuery) {
        super(id);

        IDataProvider<Catapult> dataProvider;
        if (searchQuery == null) {
            dataProvider = new CatapultListDataProvider(catapultQueryService);
        } else {
            dataProvider = new CatapultSearchDataProvider(catapultQueryService, searchQuery);
        }

        DataView<Catapult> dataView = new  DataView<Catapult>("rows", dataProvider, ITEMS_PER_PAGE) {
            @Override
            protected void populateItem(Item<Catapult> item) {
                final Catapult catapult = item.getModelObject();
                item.setModel(new CompoundPropertyModel<Catapult>(catapult));
                item.add(new Label("id"));
                item.add(new Label("name"));
                item.add(new Label("created"));
                item.add(new Label("lastModified"));
                item.add(new Link<Catapult>("delete", item.getModel()) {
                    @Override
                    public void onClick() {
                        // Detached object: Need to delete by ID!
                        catapultUpdateService.delete(catapult.getId());
                    }
                });
            }
        };

        add(dataView);
        add(new PagingNavigator("footerPaginator", dataView));
    }
}
