/*
 * Copyright 2008-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package almira.sample.web.model;

import java.io.Serializable;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.lucene.queryParser.ParseException;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;

import almira.sample.domain.DomainObject;
import almira.sample.service.ObjectQueryService;


/**
 * http://www.mail-archive.com/users@wicket.apache.org/msg14735.html
 *
 * @author 'Daniel TISCHER'
 */
public abstract class AbstractCachingDataProvider<T extends DomainObject<ID>, ID extends Serializable>
        implements IDataProvider<T> {

    private Integer size = null;
    private Iterator<T> iterator = null;

    private final ObjectQueryService<T, ID> service;
    private static final Logger LOG = Logger
            .getLogger(AbstractCachingDataProvider.class.getName());


    AbstractCachingDataProvider(ObjectQueryService<T, ID> queryService) {
        this.service = queryService;
    }

    protected abstract Iterator<T> query(int startIndex, int fetchSize);

    protected abstract int count();

    final Iterator<T> findAll(int startIndex, int fetchSize) {
        return service.findAll(startIndex, fetchSize).iterator();
    }

    final int countAll() {
        return service.count();
    }

    final Iterator<T> searchByName(String query, int startIndex, int fetchSize) {
        Iterator<T> result = null;
        try {
            result = service.searchByName(query, startIndex, fetchSize).iterator();
        } catch (ParseException e) {
            LOG.log(Level.SEVERE, "bad query", e);
        }
        return result;
    }

    final int countByName(String query) {
        int count = 0;
        try {
            count = service.countByName(query);
        } catch (ParseException e) {
            LOG.log(Level.SEVERE, "bad query", e);
        }
        return count;
    }

    @Override
    public final void detach() {
        size = null;
        iterator = null;
    }

    @Override
    public final Iterator<T> iterator(int startIndex, int fetchSize) {
        if (iterator == null) {
            iterator = query(startIndex, fetchSize);
        }
        return iterator;
    }

    @Override
    public final int size() {
        if (size == null) {
            size = count();
        }
        return size;
    }


    @edu.umd.cs.findbugs.annotations.SuppressWarnings(
          value = {"SE_INNER_CLASS"}, justification = "This is the Wicket way")

    @Override
    public final IModel<T> model(final T object) {
        return new LoadableDetachableModel<T>() {
            @Override
            protected T load() {
                return service.load(object.getId());
            }
        };
    }
}